@@ -14,7 +14,10 @@
 # limitations under the License.
 
 import re
-from collections import MutableMapping
+try:
+    from collections.abc import MutableMapping
+except ImportError:
+    from collections import MutableMapping  # py2
 from functools import partial
 
 from swift.common import header_key_dict
@@ -503,7 +506,7 @@ class InvalidURI(ErrorResponse):
         ErrorResponse.__init__(self, msg, uri=uri, *args, **kwargs)
 
 
-class KeyTooLong(ErrorResponse):
+class KeyTooLongError(ErrorResponse):
     _status = '400 Bad Request'
     _msg = 'Your key is too long.'
 
