@@ -22,7 +22,11 @@ import logging
 import logging.handlers
 import sys
 from contextlib import contextmanager, closing
-from collections import defaultdict, Iterable
+from collections import defaultdict
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable  # py2
 import itertools
 from numbers import Number
 from tempfile import NamedTemporaryFile
@@ -45,12 +49,12 @@ from six.moves import range
 from six.moves.http_client import HTTPException
 
 from swift.common import storage_policy, swob, utils
+from swift.common.memcached import MemcacheConnectionError
 from swift.common.storage_policy import (StoragePolicy, ECStoragePolicy,
                                          VALID_EC_TYPES)
 from swift.common.utils import Timestamp, md5
 from test import get_config
-# import to namespace for backward compat
-from test.debug_logger import debug_logger, DebugLogger, FakeLogger  # noqa
+from test.debug_logger import FakeLogger
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.ring import Ring, RingData, RingBuilder
 from swift.obj import server
@@ -216,7 +220,8 @@ class PatchPolicies(object):
 class FakeRing(Ring):
 
     def __init__(self, replicas=3, max_more_nodes=0, part_power=0,
-                 base_port=1000, separate_replication=False):
+                 base_port=1000, separate_replication=False,
+                 next_part_power=None, reload_time=15):
         self.serialized_path = '/foo/bar/object.ring.gz'
         self._base_port = base_port
         self.max_more_nodes = max_more_nodes
@@ -225,7 +230,9 @@ class FakeRing(Ring):
         self.separate_replication = separate_replication
         # 9 total nodes (6 more past the initial 3) is the cap, no matter if
         # this is set higher, or R^2 for R replicas
+        self.reload_time = reload_time
         self.set_replicas(replicas)
+        self._next_part_power = next_part_power
         self._reload()
 
     def has_changed(self):
@@ -394,42 +401,64 @@ class FabricatedRing(Ring):
         self._update_bookkeeping()
 
 
+def track(f):
+    def wrapper(self, *a, **kw):
+        self.calls.append(getattr(mocklib.call, f.__name__)(*a, **kw))
+        return f(self, *a, **kw)
+    return wrapper
+
+
 class FakeMemcache(object):
 
     def __init__(self):
         self.store = {}
         self.calls = []
+        self.error_on_incr = False
+        self.init_incr_return_neg = False
 
     def clear_calls(self):
         del self.calls[:]
 
-    def _called(self, method, key=None, value=None, time=None):
-        self.calls.append((method, key, value, time))
-
+    @track
     def get(self, key):
-        self._called('get', key)
         return self.store.get(key)
 
+    @property
     def keys(self):
-        self._called('keys')
-        return self.store.keys()
+        return self.store.keys
 
-    def set(self, key, value, time=0):
-        self._called('set', key, value, time)
+    @track
+    def set(self, key, value, serialize=True, time=0):
+        if serialize:
+            value = json.loads(json.dumps(value))
+        else:
+            assert isinstance(value, (str, bytes))
         self.store[key] = value
         return True
 
-    def incr(self, key, time=0):
-        self._called('incr', key, time=time)
-        self.store[key] = self.store.setdefault(key, 0) + 1
+    @track
+    def incr(self, key, delta=1, time=0):
+        if self.error_on_incr:
+            raise MemcacheConnectionError('Memcache restarting')
+        if self.init_incr_return_neg:
+            # simulate initial hit, force reset of memcache
+            self.init_incr_return_neg = False
+            return -10000000
+        self.store[key] = int(self.store.setdefault(key, 0)) + delta
+        if self.store[key] < 0:
+            self.store[key] = 0
         return self.store[key]
 
+    # tracked via incr()
+    def decr(self, key, delta=1, time=0):
+        return self.incr(key, delta=-delta, time=time)
+
     @contextmanager
     def soft_lock(self, key, timeout=0, retries=5):
         yield True
 
+    @track
     def delete(self, key):
-        self._called('delete', key)
         try:
             del self.store[key]
         except Exception:
@@ -440,6 +469,11 @@ class FakeMemcache(object):
         self.store.clear()
 
 
+# This decorator only makes sense in the context of FakeMemcache;
+# may as well clean it up now
+del track
+
+
 class FakeIterable(object):
     def __init__(self, values):
         self.next_call_count = 0
@@ -933,9 +967,15 @@ def fake_http_connect(*code_iter, **kwargs):
 
         if 'give_connect' in kwargs:
             give_conn_fn = kwargs['give_connect']
-            argspec = inspect.getargspec(give_conn_fn)
-            if argspec.keywords or 'connection_id' in argspec.args:
-                ckwargs['connection_id'] = i
+
+            if six.PY2:
+                argspec = inspect.getargspec(give_conn_fn)
+                if argspec.keywords or 'connection_id' in argspec.args:
+                    ckwargs['connection_id'] = i
+            else:
+                argspec = inspect.getfullargspec(give_conn_fn)
+                if argspec.varkw or 'connection_id' in argspec.args:
+                    ckwargs['connection_id'] = i
             give_conn_fn(*args, **ckwargs)
         etag = next(etag_iter)
         headers = next(headers_iter)
@@ -1010,6 +1050,13 @@ def mock_timestamp_now(now=None):
         yield now
 
 
+@contextmanager
+def mock_timestamp_now_with_iter(ts_iter):
+    with mocklib.patch('swift.common.utils.Timestamp.now',
+                       side_effect=ts_iter):
+        yield
+
+
 class Timeout(object):
     def __init__(self, seconds):
         self.seconds = seconds
@@ -1038,16 +1085,29 @@ def requires_o_tmpfile_support_in_tmp(func):
 
 class StubResponse(object):
 
-    def __init__(self, status, body=b'', headers=None, frag_index=None):
+    def __init__(self, status, body=b'', headers=None, frag_index=None,
+                 slowdown=None):
         self.status = status
         self.body = body
         self.readable = BytesIO(body)
+        try:
+            self._slowdown = iter(slowdown)
+        except TypeError:
+            self._slowdown = iter([slowdown])
         self.headers = HeaderKeyDict(headers)
         if frag_index is not None:
             self.headers['X-Object-Sysmeta-Ec-Frag-Index'] = frag_index
         fake_reason = ('Fake', 'This response is a lie.')
         self.reason = swob.RESPONSE_REASONS.get(status, fake_reason)[0]
 
+    def slowdown(self):
+        try:
+            wait = next(self._slowdown)
+        except StopIteration:
+            wait = None
+        if wait is not None:
+            eventlet.sleep(wait)
+
     def nuke_from_orbit(self):
         if hasattr(self, 'swift_conn'):
             self.swift_conn.close()
@@ -1061,8 +1121,13 @@ class StubResponse(object):
         return self.headers.items()
 
     def read(self, amt=0):
+        self.slowdown()
         return self.readable.read(amt)
 
+    def readline(self, size=-1):
+        self.slowdown()
+        return self.readable.readline(size)
+
     def __repr__(self):
         info = ['Status: %s' % self.status]
         if self.headers:
