@@ -574,13 +574,10 @@ Shard Ranges (3):
             print_item_locations(None, partition=part, policy_name='zero',
                                  swift_dir=self.testdir)
         exp_part_msg = 'Partition\t%s' % part
-        exp_acct_msg = 'Account  \tNone'
-        exp_cont_msg = 'Container\tNone'
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_part_msg, out.getvalue())
-        self.assertIn(exp_acct_msg, out.getvalue())
-        self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Account', out.getvalue())
+        self.assertNotIn('Container', out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_dashed_ring_name_partition(self):
         out = StringIO()
@@ -590,13 +587,10 @@ Shard Ranges (3):
                                  ring_name='foo-bar', partition=part,
                                  swift_dir=self.testdir)
         exp_part_msg = 'Partition\t%s' % part
-        exp_acct_msg = 'Account  \tNone'
-        exp_cont_msg = 'Container\tNone'
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_part_msg, out.getvalue())
-        self.assertIn(exp_acct_msg, out.getvalue())
-        self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Account', out.getvalue())
+        self.assertNotIn('Container', out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_account_with_ring(self):
         out = StringIO()
@@ -610,11 +604,9 @@ Shard Ranges (3):
                       'but ring not named "account"'
         self.assertIn(exp_warning, out.getvalue())
         exp_acct_msg = 'Account  \t%s' % account
-        exp_cont_msg = 'Container\tNone'
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_acct_msg, out.getvalue())
-        self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Container', out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_account_no_ring(self):
         out = StringIO()
@@ -623,11 +615,9 @@ Shard Ranges (3):
             print_item_locations(None, account=account,
                                  swift_dir=self.testdir)
         exp_acct_msg = 'Account  \t%s' % account
-        exp_cont_msg = 'Container\tNone'
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_acct_msg, out.getvalue())
-        self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Container', out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_account_container_ring(self):
         out = StringIO()
@@ -639,10 +629,9 @@ Shard Ranges (3):
                                  container=container)
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_acct_msg, out.getvalue())
         self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_account_container_no_ring(self):
         out = StringIO()
@@ -653,10 +642,9 @@ Shard Ranges (3):
                                  container=container, swift_dir=self.testdir)
         exp_acct_msg = 'Account  \t%s' % account
         exp_cont_msg = 'Container\t%s' % container
-        exp_obj_msg = 'Object   \tNone'
         self.assertIn(exp_acct_msg, out.getvalue())
         self.assertIn(exp_cont_msg, out.getvalue())
-        self.assertIn(exp_obj_msg, out.getvalue())
+        self.assertNotIn('Object', out.getvalue())
 
     def test_print_item_locations_account_container_object_ring(self):
         out = StringIO()
@@ -768,59 +756,59 @@ Shard Ranges (3):
     def test_parse_get_node_args(self):
         # Capture error messages
         # (without any parameters)
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = ''
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
         # a
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'a'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
         # a c
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'a c'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
         # a c o
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'a c o'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
 
         # a/c
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'a/c'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
         # a/c/o
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'a/c/o'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
 
         # account container junk/test.ring.gz
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'account container junk/test.ring.gz'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
 
         # account container object junk/test.ring.gz
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'account container object junk/test.ring.gz'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Need to specify policy_name or <ring.gz>',
                                  parse_get_node_args, options, args.split())
 
         # object.ring.gz(without any arguments i.e. a c o)
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'object.ring.gz'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Ring file does not exist',
@@ -828,55 +816,55 @@ Shard Ranges (3):
 
         # Valid policy
         # -P zero
-        options = Namespace(policy_name='zero', partition=None)
+        options = Namespace(policy_name='zero', partition=None, quoted=None)
         args = ''
         self.assertRaisesMessage(InfoSystemExit,
                                  'No target specified',
                                  parse_get_node_args, options, args.split())
         # -P one a/c/o
-        options = Namespace(policy_name='one', partition=None)
+        options = Namespace(policy_name='one', partition=None, quoted=None)
         args = 'a/c/o'
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertEqual(args, ['a', 'c', 'o'])
         # -P one account container photos/cat.jpg
-        options = Namespace(policy_name='one', partition=None)
+        options = Namespace(policy_name='one', partition=None, quoted=None)
         args = 'account container photos/cat.jpg'
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertEqual(args, ['account', 'container', 'photos/cat.jpg'])
         # -P one account/container/photos/cat.jpg
-        options = Namespace(policy_name='one', partition=None)
+        options = Namespace(policy_name='one', partition=None, quoted=None)
         args = 'account/container/photos/cat.jpg'
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertEqual(args, ['account', 'container', 'photos/cat.jpg'])
         # -P one account/container/junk/test.ring.gz(object endswith 'ring.gz')
-        options = Namespace(policy_name='one', partition=None)
+        options = Namespace(policy_name='one', partition=None, quoted=None)
         args = 'account/container/junk/test.ring.gz'
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertEqual(args, ['account', 'container', 'junk/test.ring.gz'])
         # -P two a c o hooya
-        options = Namespace(policy_name='two', partition=None)
+        options = Namespace(policy_name='two', partition=None, quoted=None)
         args = 'a c o hooya'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Invalid arguments',
                                  parse_get_node_args, options, args.split())
         # -P zero -p 1
-        options = Namespace(policy_name='zero', partition='1')
+        options = Namespace(policy_name='zero', partition='1', quoted=None)
         args = ''
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertFalse(args)
         # -P one -p 1 a/c/o
-        options = Namespace(policy_name='one', partition='1')
+        options = Namespace(policy_name='one', partition='1', quoted=None)
         args = 'a/c/o'
         ring_path, args = parse_get_node_args(options, args.split())
         self.assertIsNone(ring_path)
         self.assertEqual(args, ['a', 'c', 'o'])
         # -P two -p 1 a c o hooya
-        options = Namespace(policy_name='two', partition='1')
+        options = Namespace(policy_name='two', partition='1', quoted=None)
         args = 'a c o hooya'
         self.assertRaisesMessage(InfoSystemExit,
                                  'Invalid arguments',
@@ -930,7 +918,7 @@ Shard Ranges (3):
 
         # Mock tests
         # /etc/swift/object.ring.gz(without any arguments i.e. a c o)
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = '/etc/swift/object.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -940,7 +928,7 @@ Shard Ranges (3):
                 parse_get_node_args, options, args.split())
         # Similar ring_path and arguments
         # /etc/swift/object.ring.gz /etc/swift/object.ring.gz
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = '/etc/swift/object.ring.gz /etc/swift/object.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -948,7 +936,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, '/etc/swift/object.ring.gz')
         self.assertEqual(args, ['etc', 'swift', 'object.ring.gz'])
         # /etc/swift/object.ring.gz a/c/etc/swift/object.ring.gz
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = '/etc/swift/object.ring.gz a/c/etc/swift/object.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -957,7 +945,7 @@ Shard Ranges (3):
         self.assertEqual(args, ['a', 'c', 'etc/swift/object.ring.gz'])
         # Invalid path as mentioned in BUG#1539275
         # /etc/swift/object.tar.gz account container object
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = '/etc/swift/object.tar.gz account container object'
         self.assertRaisesMessage(
             InfoSystemExit,
@@ -965,7 +953,7 @@ Shard Ranges (3):
             parse_get_node_args, options, args.split())
 
         # object.ring.gz a/
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a/'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -973,7 +961,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a'])
         # object.ring.gz a/c
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a/c'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -981,7 +969,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c'])
         # object.ring.gz a/c/o
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a/c/o'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -989,7 +977,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o'])
         # object.ring.gz a/c/o/junk/test.ring.gz
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a/c/o/junk/test.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -997,7 +985,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o/junk/test.ring.gz'])
         # object.ring.gz a
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1005,7 +993,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a'])
         # object.ring.gz a c
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a c'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1013,7 +1001,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c'])
         # object.ring.gz a c o
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a c o'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1021,7 +1009,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o'])
         # object.ring.gz a c o blah blah
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a c o blah blah'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1030,7 +1018,7 @@ Shard Ranges (3):
                 'Invalid arguments',
                 parse_get_node_args, options, args.split())
         # object.ring.gz a/c/o/blah/blah
-        options = Namespace(policy_name=None)
+        options = Namespace(policy_name=None, quoted=None)
         args = 'object.ring.gz a/c/o/blah/blah'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1039,7 +1027,7 @@ Shard Ranges (3):
         self.assertEqual(args, ['a', 'c', 'o/blah/blah'])
 
         # object.ring.gz -p 1
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1047,7 +1035,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertFalse(args)
         # object.ring.gz -p 1 a c o
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz a c o'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1055,7 +1043,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o'])
         # object.ring.gz -p 1 a c o forth_arg
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz a c o forth_arg'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1064,7 +1052,7 @@ Shard Ranges (3):
                 'Invalid arguments',
                 parse_get_node_args, options, args.split())
         # object.ring.gz -p 1 a/c/o
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz a/c/o'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1072,7 +1060,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o'])
         # object.ring.gz -p 1 a/c/junk/test.ring.gz
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz a/c/junk/test.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1080,7 +1068,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'junk/test.ring.gz'])
         # object.ring.gz -p 1 a/c/photos/cat.jpg
-        options = Namespace(policy_name=None, partition='1')
+        options = Namespace(policy_name=None, partition='1', quoted=None)
         args = 'object.ring.gz a/c/photos/cat.jpg'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1089,7 +1077,7 @@ Shard Ranges (3):
         self.assertEqual(args, ['a', 'c', 'photos/cat.jpg'])
 
         # --all object.ring.gz a
-        options = Namespace(all=True, policy_name=None)
+        options = Namespace(all=True, policy_name=None, quoted=None)
         args = 'object.ring.gz a'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1097,7 +1085,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a'])
         # --all object.ring.gz a c
-        options = Namespace(all=True, policy_name=None)
+        options = Namespace(all=True, policy_name=None, quoted=None)
         args = 'object.ring.gz a c'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1105,7 +1093,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c'])
         # --all object.ring.gz a c o
-        options = Namespace(all=True, policy_name=None)
+        options = Namespace(all=True, policy_name=None, quoted=None)
         args = 'object.ring.gz a c o'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1113,7 +1101,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['a', 'c', 'o'])
         # object.ring.gz account container photos/cat.jpg
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'object.ring.gz account container photos/cat.jpg'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1121,7 +1109,7 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['account', 'container', 'photos/cat.jpg'])
         # object.ring.gz /account/container/photos/cat.jpg
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'object.ring.gz account/container/photos/cat.jpg'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1130,7 +1118,7 @@ Shard Ranges (3):
         self.assertEqual(args, ['account', 'container', 'photos/cat.jpg'])
         # Object name ends with 'ring.gz'
         # object.ring.gz /account/container/junk/test.ring.gz
-        options = Namespace(policy_name=None, partition=None)
+        options = Namespace(policy_name=None, partition=None, quoted=None)
         args = 'object.ring.gz account/container/junk/test.ring.gz'
         with mock.patch('swift.cli.info.os.path.exists') as exists:
             exists.return_value = True
@@ -1138,6 +1126,32 @@ Shard Ranges (3):
         self.assertEqual(ring_path, 'object.ring.gz')
         self.assertEqual(args, ['account', 'container', 'junk/test.ring.gz'])
 
+        # Object name has special characters
+        # object.ring.gz /account/container/obj\nwith%0anewline
+        options = Namespace(policy_name=None, partition=None, quoted=None)
+        args = ['object.ring.gz', 'account/container/obj\nwith%0anewline']
+        with mock.patch('swift.cli.info.os.path.exists') as exists:
+            exists.return_value = True
+            ring_path, args = parse_get_node_args(options, args)
+        self.assertEqual(ring_path, 'object.ring.gz')
+        self.assertEqual(args, ['account', 'container', 'obj\nwith%0anewline'])
+
+        options = Namespace(policy_name=None, partition=None, quoted=True)
+        args = ['object.ring.gz', 'account/container/obj\nwith%0anewline']
+        with mock.patch('swift.cli.info.os.path.exists') as exists:
+            exists.return_value = True
+            ring_path, args = parse_get_node_args(options, args)
+        self.assertEqual(ring_path, 'object.ring.gz')
+        self.assertEqual(args, ['account', 'container', 'obj\nwith\nnewline'])
+
+        options = Namespace(policy_name=None, partition=None, quoted=False)
+        args = ['object.ring.gz', 'account/container/obj\nwith%0anewline']
+        with mock.patch('swift.cli.info.os.path.exists') as exists:
+            exists.return_value = True
+            ring_path, args = parse_get_node_args(options, args)
+        self.assertEqual(ring_path, 'object.ring.gz')
+        self.assertEqual(args, ['account', 'container', 'obj\nwith%0anewline'])
+
 
 class TestPrintObj(TestCliInfoBase):
 
@@ -1265,7 +1279,7 @@ class TestPrintObjFullMeta(TestCliInfoBase):
         exp_curl = (
             'curl -g -I -XHEAD '
             '"http://{host}:{port}/{device}/{part}/AUTH_admin/c/obj" '
-            '-H "X-Backend-Storage-Policy-Index: 2"').format(
+            '-H "X-Backend-Storage-Policy-Index: 2" --path-as-is').format(
                 host=node['ip'],
                 port=node['port'],
                 device=node['device'],
@@ -1306,7 +1320,8 @@ class TestPrintObjFullMeta(TestCliInfoBase):
         exp_curl = (
             'curl -g -I -XHEAD '
             '"http://[{host}]:{port}'
-            '/{device}/{part}/AUTH_admin/c/obj" ').format(
+            '/{device}/{part}/AUTH_admin/c/obj" '
+            '-H "X-Backend-Storage-Policy-Index: 3" --path-as-is').format(
                 host=node['ip'],
                 port=node['port'],
                 device=node['device'],
