@@ -25,7 +25,8 @@ from swift.common.middleware import symlink, copy, versioned_writes, \
     listing_formats
 from swift.common.swob import Request
 from swift.common.request_helpers import get_reserved_name
-from swift.common.utils import MD5_OF_EMPTY_STRING, get_swift_info
+from swift.common.utils import MD5_OF_EMPTY_STRING
+from swift.common.registry import get_swift_info
 from test.unit.common.middleware.helpers import FakeSwift
 from test.unit.common.middleware.test_versioned_writes import FakeCache
 
@@ -403,6 +404,7 @@ class TestSymlinkMiddleware(TestSymlinkMiddlewareBase):
         req = Request.blank('/v1/a/c/symlink?symlink=get', method='GET')
         status, headers, body = self.call_sym(req)
         self.assertEqual(status, '200 OK')
+        self.assertIsInstance(headers, list)
         self.assertIn(('X-Symlink-Target', 'c1/o'), headers)
         self.assertNotIn('X-Symlink-Target-Account', dict(headers))
 
